﻿using Syncfusion.EJ.Export;
using Syncfusion.JavaScript.Models;
using Syncfusion.XlsIO;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data.SqlClient;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;

namespace SpreadsheetDBSample.Controllers
{
    public class HomeController : Controller
    {
        //
        // GET: /Home/
        private string connetionString = ConfigurationManager.ConnectionStrings["SpreadDBConnectionString"].ConnectionString;
        public ActionResult Index()
        {
            ViewBag.FileNameList = GetDDdata();
            return View();
        }

        //Save changes only
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SaveChangesToDatabase(string fileName, string editedValues)
        {
            JavaScriptSerializer serialize = new JavaScriptSerializer();
            List<CellDetail> editedData = serialize.Deserialize<List<CellDetail>>(editedValues);

            MemoryStream dataStream = new MemoryStream();
            SqlConnection sqlCon = new SqlConnection(connetionString);
            SqlCommand sqlComm = new SqlCommand("SELECT * FROM [dbo].[Table] WHERE [FileName] = '" + fileName + "'", sqlCon);
            sqlCon.Open();
            SqlDataReader sqlDR = sqlComm.ExecuteReader();
            if (sqlDR.Read())
            {
                dataStream = new MemoryStream((byte[])sqlDR.GetValue(1)); // Get Stream values from DB.
            }
            sqlDR.Close();
            ExcelEngine excelEngine = new ExcelEngine();
            IApplication application = excelEngine.Excel;
            IWorkbook workbook = application.Workbooks.Open(dataStream);
            IWorksheet sheet;
            for (int i = 0; i < editedData.Count; i++)
            {
                CellDetail data = editedData[i];
                sheet = workbook.Worksheets[data.SheetIndex - 1];
                // C# backend edit process by using XlsIo Library.
                sheet.Rows[data.CellIndex.RowIndex].Cells[data.CellIndex.ColIndex].Value = data.Value;
            }
            MemoryStream newDataStream = new MemoryStream();
            workbook.Version = ExcelVersion.Excel2013;
            workbook.Application.DefaultVersion = ExcelVersion.Excel2013;
            workbook.SaveAs(newDataStream);

            SqlCommand updateComm = new SqlCommand("UPDATE [dbo].[Table] SET FileData=@nFileData WHERE FileName=@nFileName", sqlCon);            
            byte[] dataBytes = newDataStream.ToArray();
            updateComm.Parameters.AddWithValue("@nFileName", fileName);
            updateComm.Parameters.AddWithValue("@nFileData", dataBytes);
            updateComm.ExecuteNonQuery();
            sqlCon.Close();
            return Content("Successfully updated");
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SaveAndUpdateToDatabase(string fileName, string sheetModel, string sheetData)
        {
            MemoryStream dataStream = (MemoryStream)Spreadsheet.Save(sheetModel, sheetData, ExportFormat.XLSX, ExcelVersion.Excel2013);            
            byte[] dataBytes = dataStream.ToArray();
            SqlConnection sqlCon = new SqlConnection(connetionString);
            sqlCon.Open();
            SqlCommand selectComm = new SqlCommand("SELECT * FROM [dbo].[Table] WHERE [FileName] = '" + fileName + "'", sqlCon);
            var hasFileInDB = selectComm.ExecuteScalar();
            if (hasFileInDB == null)
            {
                SqlCommand sqlComm = new SqlCommand("INSERT INTO [dbo].[Table]([FileName], [FileData]) VALUES (@FileName, @FileData)", sqlCon);
                sqlComm.Parameters.AddWithValue("@FileName", fileName);
                sqlComm.Parameters.AddWithValue("@FileData", dataBytes);
                sqlComm.ExecuteNonQuery();
            }
            else
            {
                SqlCommand updateComm = new SqlCommand("UPDATE [dbo].[Table] SET FileData=@nFileData WHERE FileName=@nFileName", sqlCon); // Update edited data to DB.
                updateComm.CommandTimeout = 0;
                updateComm.Parameters.AddWithValue("@nFileName", fileName);
                updateComm.Parameters.AddWithValue("@nFileData", dataBytes);
                updateComm.ExecuteNonQuery();
            }
            sqlCon.Close();
            return Content(fileName);
        }

        //File Open from database
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult OpenFileFromDB(string filename)
        {
            ImportRequest importRequest = new ImportRequest();
            SqlConnection sqlCon = new SqlConnection(connetionString);
            SqlCommand sqlComm = new SqlCommand("SELECT * FROM [dbo].[Table] WHERE [FileName] = '" + filename + "'", sqlCon);
            sqlCon.Open();
            SqlDataReader sqlDR = sqlComm.ExecuteReader();
            if (sqlDR.Read())
            {
                importRequest.FileStream = new MemoryStream((byte[])sqlDR.GetValue(1)); // Get binary values from DB.
            }
            sqlCon.Close();
            return Content(Spreadsheet.Open(importRequest));
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public void ExcelExport(string sheetModel, string sheetData, string password)
        {
            if (String.IsNullOrEmpty(password))
                Spreadsheet.Save(sheetModel, sheetData, "sample", ExportFormat.XLSX, ExcelVersion.Excel2013);
            else
                Spreadsheet.Save(sheetModel, sheetData, "sample", ExportFormat.XLSX, ExcelVersion.Excel2013, password);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public void CsvExport(string sheetModel, string sheetData)
        {
            Spreadsheet.Save(sheetModel, sheetData, "sample", ExportFormat.CSV);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public void PdfExport(string sheetModel, string sheetData)
        {
            Spreadsheet.Save(sheetModel, sheetData, "sample", ExportFormat.PDF);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Import(ImportRequest importRequest)
        {
            return Content(Spreadsheet.Open(importRequest));
        }

        public List<SelectListItem> GetDDdata()
        {
            List<SelectListItem> ddDataSource = new List<SelectListItem>();
            ddDataSource.Add(new SelectListItem() { Text = "", Value = "", Selected = true });
            SqlConnection sqlCon = new SqlConnection(connetionString);
            SqlCommand sqlComm = new SqlCommand("SELECT [FileName] FROM [dbo].[Table]", sqlCon);
            sqlCon.Open();
            SqlDataReader sqlDR = sqlComm.ExecuteReader();
            while (sqlDR.Read())
            {
                var fileName = sqlDR.GetValue(0).ToString();
                ddDataSource.Add(new SelectListItem { Value = fileName, Text = fileName.Replace(".xlsx", "") });
            }
            sqlCon.Close();
            return ddDataSource;
        }
    }
}
